#include "clock.hpp"

#include <type_traits>

#include "eink/eink.hpp"
#include "flash/flash.hpp"
#include "io/io.hpp"
#include "uart/uart.hpp"

#include "image.hpp"

using uart_t = uart::Uart0<uart::Config<115200>>;
REGISTER_UART0_INT_VECTORS(uart_t);

using spi_t = eink::Spi<io::P::C4, io::P::C5, io::P::C3, io::P::C2>;
using eink_t = eink::Eink<200, 200, spi_t, io::P::C1, io::P::C0>;

void dumpOTP(eink_t &einkDisplay, uart_t &serial)
{
	constexpr auto printWidth = 16;
	auto printedCnt = std::size_t{0};

	einkDisplay.dumpOTP([&serial, &printedCnt](const auto &data) {
		constexpr auto printAddress = true;

		if (printedCnt > 0 && printedCnt % printWidth == 0) {
			serial << F("\r\n");
		}

		if (printAddress && printedCnt % printWidth == 0) {
			serial << F("0x");
			serial.txNumber<std::size_t, 16, 4>(printedCnt);
			serial << F(": ");
		}

		serial << F("0x");
		serial.txNumber<std::remove_cvref_t<decltype(data)>, 16, 2>(data);
		serial << F(", ");

		++printedCnt;
	});
	serial << F("\r\n");
}

int main()
{
	uart_t serial;
	serial.init();

	sei();

	serial << F("e-Paper demo") << F("\r\n");

	auto einkDisplay = eink_t{};

	serial << F("e-Paper init") << F("\r\n");
	einkDisplay.init();

	serial << F("e-Paper clear") << F("\r\n");
	einkDisplay.clear();

	serial << F("e-Paper draw") << F("\r\n");
	einkDisplay.draw(RLE_IMAGE);

	serial << F("e-Paper dump OTP") << F("\r\n");
	dumpOTP(einkDisplay, serial);

	serial << F("e-Paper sleep") << F("\r\n");
	einkDisplay.sleep();

	while (true) {
		serial << F("e-Paper running") << F("\r\n");
		_delay_ms(1000);
	}

	return 0;
}
